@@ -6,7 +6,7 @@
 // This file may be distributed under the terms of the GNU LGPLv3 license.
 
 /*
- * Copyright (c) 2010-2013 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  */
 
 #include "acpi.h" // struct rsdp_descriptor
@@ -19,6 +19,7 @@
 #include "config.h" // CONFIG_*
 #include "paravirt.h" // RamSize
 #include "dev-q35.h"
+#include "memmap.h"
 
 #include "acpi-dsdt.hex"
 #include "config.h"
@@ -196,7 +197,7 @@ build_madt(void)
     io_apic->interrupt = cpu_to_le32(0);
 
     struct madt_intsrcovr *intsrcovr = (void*)&io_apic[1];
-    if (CONFIG_IRQ0_OVERRIDE == 1 ||  romfile_loadint("etc/irq0-override", 0)) {
+    if (romfile_loadint("etc/irq0-override", 0)) {
         memset(intsrcovr, 0, sizeof(*intsrcovr));
         intsrcovr->type   = APIC_XRUPT_OVERRIDE;
         intsrcovr->length = sizeof(*intsrcovr);
@@ -436,7 +437,7 @@ build_ssdt(void)
     *(ssdt_ptr++) = '0';
 
     // build Device object for each slot
-    u32 rmvc_pcrm = inl(PCI_RMV_BASE);
+    u32 rmvc_pcrm = CONFIG_DISABLE_PCI_HOTPLUG ? 0 : inl(PCI_RMV_BASE);
     for (i=1; i<PCI_SLOTS; i++) {
         u32 eject = rmvc_pcrm & (0x1 << i);
         memcpy(ssdt_ptr, PCIHP_AML, PCIHP_SIZEOF);
@@ -630,6 +631,24 @@ static const struct pci_device_id acpi_find_tbl[] = {
 
 struct rsdp_descriptor *RsdpAddr;
 
+void
+update_e820_based_on_mcfg(struct acpi_table_header *table)
+{
+	int count;
+	int i;
+
+    struct acpi_table_mcfg *mcfg = (struct acpi_table_mcfg *)table;
+    count = (table->length - sizeof(struct acpi_table_header))
+        / sizeof(struct acpi_mcfg_allocation);
+
+    for (i = 0; i < count; i++) {
+        add_e820(mcfg->allocation[i].address,
+                 (mcfg->allocation[i].end_bus_number + 1
+                  - mcfg->allocation[i].start_bus_number) * 256 * 4096,
+                 E820_RESERVED);
+    }
+}
+
 #define MAX_ACPI_TABLES 20
 void
 acpi_setup(void)
@@ -684,6 +703,9 @@ acpi_setup(void)
         } else {
             ACPI_INIT_TABLE(table);
         }
+        if (table->signature == MCFG_SIGNATURE) {
+            update_e820_based_on_mcfg(table);
+        }
         if (tbl_idx == MAX_ACPI_TABLES) {
             warn_noalloc();
             break;
