@@ -14,17 +14,16 @@
 #define BUILD_APPNAME6 "BOCHS "
 #define BUILD_APPNAME4 "BXPC"
 
-// Force to generate ACPI tables even if PIIX4 power management device
-// does not exists.
+// Configuration definissions for the VMM.
 #define CONFIG_FORCE_ACPI 1
 #define CONFIG_NO_PCI_SETUP 1
-#define CONFIG_IRQ0_OVERRIDE 1
 #define CONFIG_FORCE_APIC_PHYSICAL_DESTINATION_MODE 1
 #define CONFIG_USE_TSC_FOR_INT15_86 1
 #define CONFIG_GET_ACPI_PM_BASE_FROM_RTC 1
 #define CONFIG_DISABLE_APM_PORT 1
-#define CONFIG_SCREEN_AND_DEBUG 0
 #define CONFIG_AHCI_HBA_RESET 0
+#define CONFIG_DISABLE_PCI_HOTPLUG 1
+#define CONFIG_INT10_0E_TO_DPRINT 0
 
 // Maximum number of map entries in the e820 map
 #define BUILD_MAX_E820 32
