@@ -49,7 +49,7 @@ handle_10(struct bregs *regs)
     debug_enter(regs, DEBUG_HDL_10);
     // dont do anything, since the VGA BIOS handles int10h requests
 
-    if (regs->ah == 0x0e) {
+    if (CONFIG_INT10_0E_TO_DPRINT && regs->ah == 0x0e) {
         /* Teletype output */
         char c;
         c = regs->al;
