@@ -174,11 +174,15 @@ static struct rsdp rsdp_copy;
 static struct acpi_parser parsers[PARSERS_MAX_NUM];
 static uint parsers_num = 0;
 
+static bool pm1a_cnt_found = false;
+static ioport_t pm1a_cnt_ioaddr;
+static bool pm1b_cnt_found = false;
+static ioport_t pm1b_cnt_ioaddr;
 static bool reset_reg_found = false;
 static ioport_t reset_reg_ioaddr;
 static u8 reset_value;
 
-unsigned char acpi_dsdt_system_state[6][5];
+struct acpi_sleep acpi_sleep[6];
 
 static u8
 acpi_checksum (void *p, int len)
@@ -363,6 +367,89 @@ debug_dump (void *p, int len)
 	}
 }
 
+void
+acpi_poweroff (void)
+{
+	u32 data, typx;
+
+	if (!pm1a_cnt_found)
+		return;
+	if (!acpi_sleep[5].valid)
+		return;
+
+	ACPI_DBG("acpi_poweroff\n");
+	typx = acpi_sleep[5].type[0] << PM1_CNT_SLP_TYPX_SHIFT;
+
+	asm_inl (pm1a_cnt_ioaddr, &data);
+	data &= ~PM1_CNT_SLP_TYPX_MASK;
+	data |= typx & PM1_CNT_SLP_TYPX_MASK;
+	data |= PM1_CNT_SLP_EN_BIT;
+	asm_outl (pm1a_cnt_ioaddr, data);
+
+	if (pm1b_cnt_found) {
+		typx = acpi_sleep[5].type[1] << PM1_CNT_SLP_TYPX_SHIFT;
+		asm_inl (pm1b_cnt_ioaddr, &data);
+		data &= ~PM1_CNT_SLP_TYPX_MASK;
+		data |= typx & PM1_CNT_SLP_TYPX_MASK;
+		data |= PM1_CNT_SLP_EN_BIT;
+		asm_outl (pm1b_cnt_ioaddr, data);
+	}
+
+	freeze();
+}
+
+static void
+acpi_pm1_sleep (ioport_t port, u32 val)
+{
+	u8 type;
+
+	type = (val & PM1_CNT_SLP_TYPX_MASK) >> PM1_CNT_SLP_TYPX_SHIFT;
+	ACPI_DBG("acpi_pm1_sleep 0x%x 0x%x\n", val, type);
+
+	if (acpi_sleep[5].valid == false) {
+		return;
+	}
+	 
+	if ((port == pm1a_cnt_ioaddr && acpi_sleep[5].type[0] == type) ||
+	    (port == pm1b_cnt_ioaddr && acpi_sleep[5].type[1] == type)) {
+		ACPI_DBG("shutdown\n");
+		vm_shutdown();
+	}
+}
+
+static int
+acpi_pm1_cnt_reg (iotype_t type, ioport_t port, void *data)
+{
+	u32 val;
+
+	ACPI_DBG("acpi_pm1_cnt_reg type 0x%x port 0x%x\n", type, port);
+	switch (type) {
+	case IOTYPE_OUTB:
+		val = *(u8 *)data;
+		break;
+	case IOTYPE_OUTW:
+		val = *(u16 *)data;
+		break;
+	case IOTYPE_OUTL:
+		val = *(u32 *)data;
+		break;
+	default:
+		return 0; /* pass */
+	}
+	if ((val & PM1_CNT_SLP_EN_BIT) == 0) {
+		return 0; /* pass */
+	}
+	/*
+	 * If shutdown, the following function never returns.
+ 	 */
+	acpi_pm1_sleep (port, val);
+#ifdef DISABLE_SLEEP
+	return 1; /* emulated */
+#else
+	return 0; /* pass */
+#endif
+}
+
 static int
 acpi_reset_reg (iotype_t type, ioport_t port, void *data)
 {
@@ -403,10 +490,68 @@ acpi_iohook (void)
 		return;
 	}
 
+	if (pm1a_cnt_found)
+		set_iofunc (pm1a_cnt_ioaddr, acpi_pm1_cnt_reg);
+	if (pm1b_cnt_found)
+		set_iofunc (pm1b_cnt_ioaddr, acpi_pm1_cnt_reg);
+
 	if (reset_reg_found && reset_reg_ioaddr != 0x64)
 		set_iofunc(reset_reg_ioaddr, acpi_reset_reg);
 }
 
+static void
+get_pm1_cnt_ioaddr (struct facp *q)
+{
+	if (IS_STRUCT_SIZE_OK (q->header.length, q, q->x_pm1a_cnt_blk) &&
+	    q->x_pm1a_cnt_blk.address != 0) {
+		if (q->x_pm1a_cnt_blk.address_space_id !=
+		    ADDRESS_SPACE_ID_IO)
+			printf("X_PM1a_CNT_BLK is not I/O address\n");
+		if (q->x_pm1a_cnt_blk.address > 0xffff) {
+			printf("X_PM1a_CNT_BLK > 0xffff\n");
+		} else {
+			pm1a_cnt_ioaddr = q->x_pm1a_cnt_blk.address;
+			pm1a_cnt_found = true;
+			ACPI_DBG("PM1a control port (x) x%X\n",
+				 pm1a_cnt_ioaddr);
+		}
+	} else if (IS_STRUCT_SIZE_OK (q->header.length, q, q->pm1a_cnt_blk)) {
+		if (q->pm1a_cnt_blk > 0xFFFF) {
+			printf("pm1a_cnt_blk > 0xffff\n");
+		} else if (q->pm1a_cnt_blk == 0) {
+			printf("pm1a_cnt_blk is 0\n");
+		} else {
+			pm1a_cnt_ioaddr = q->pm1a_cnt_blk;
+			pm1a_cnt_found = true;
+			ACPI_DBG("PM1a control port is 0x%X\n",
+				 pm1a_cnt_ioaddr);
+		}
+	}
+	if (IS_STRUCT_SIZE_OK (q->header.length, q, q->x_pm1b_cnt_blk) &&
+	    (q->x_pm1b_cnt_blk.address != 0)) {
+		if (q->x_pm1b_cnt_blk.address_space_id !=
+		    ADDRESS_SPACE_ID_IO)
+			printf("X_PM1b_CNT_BLK is not I/O address\n");
+		if (q->x_pm1b_cnt_blk.address > 0xffff) {
+			printf("X_PM1b_CNT_BLK > 0xffff\n");
+		} else {
+			pm1b_cnt_ioaddr = q->x_pm1b_cnt_blk.address;
+			pm1b_cnt_found = true;
+			ACPI_DBG("PM1b control port (x) is 0x%X\n",
+				 pm1b_cnt_ioaddr);
+		}
+	} else if (IS_STRUCT_SIZE_OK (q->header.length, q, q->pm1b_cnt_blk)) {
+		if (q->pm1b_cnt_blk > 0xFFFF) {
+			printf("pm1b_cnt_blk > 0xffff\n");
+		} else if (q->pm1b_cnt_blk != 0) {
+			pm1b_cnt_ioaddr = q->pm1b_cnt_blk;
+			pm1b_cnt_found = true;
+			ACPI_DBG("PM1b control port is 0x%X\n",
+				 pm1b_cnt_ioaddr);
+		}
+	}
+}
+
 static void
 get_reset_reg_ioaddr (struct facp *facp)
 {
@@ -470,6 +615,7 @@ parse_facp(void *table, u32 length)
 #ifdef ACPI_DSDT
 	acpi_dsdt_parse(facp->dsdt);
 #endif
+	get_pm1_cnt_ioaddr(facp);
 	get_reset_reg_ioaddr(facp);
 	if (0)
 		debug_dump(facp, facp->header.length);
