@@ -463,7 +463,11 @@ apic_mmio_handler(void *data, phys_t gphys, bool wr, void *buf,
 
 	switch(gphys - APIC_BASE_ADDRESS) {
 	case APIC_APIC_ID_OFFSET:
+		emulated = apic_mmio_ro(data, gphys, wr, buf, len, flags);
+		break;
 	case APIC_LVT_LINT0_OFFSET:
+	case APIC_LVT_LINT1_OFFSET:
+		/* apic_maskall masks lint0 and lint1. So leave them. */
 		emulated = apic_mmio_ro(data, gphys, wr, buf, len, flags);
 		break;
 	case APIC_EOI_OFFSET:
