@@ -142,6 +142,9 @@
 #define MSR_AMD_VM_CR_SVMDIS_BIT	0x10
 #define MSR_AMD_VM_HSAVE_PA		0xC0010117
 
+/* 16-Bit Control Fields */
+#define VMCS_VPID			0x0
+
 /* 16-Bit Guest-State Fields */
 #define VMCS_GUEST_ES_SEL		0x800
 #define VMCS_GUEST_CS_SEL		0x802
@@ -338,6 +341,7 @@
 #define VMCS_PROC_BASED_VMEXEC_CTL_PAUSEEXIT_BIT	0x40000000
 #define VMCS_PROC_BASED_VMEXEC_CTL_SEC_CTL_BIT		0x80000000
 #define VMCS_PROC_BASED_VMEXEC_CTL2_EPT_BIT		0x2
+#define VMCS_PROC_BASED_VMEXEC_CTL2_VPID_BIT		0x20
 #define VMCS_PROC_BASED_VMEXEC_UNRESTRICTED_GUEST_BIT	0x80
 #define VMCS_GUEST_ACTIVITY_STATE_ACTIVE	0x0
 #define VMCS_GUEST_ACTIVITY_STATE_HLT		0x1
@@ -600,6 +604,7 @@
 
 #define CR3_PWT_BIT			0x8
 #define CR3_PCD_BIT			0x10
+#define CR3_NO_INVALIDATE		0x8000000000000000
 
 #define CR4_VME_BIT			0x1
 #define CR4_PVI_BIT			0x2
