@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Copyright (c) 2010-2013 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  */
 
 /* MMU emulation, Shadow Page Tables (SPT) */
@@ -68,10 +68,6 @@ struct map_page_data2 {
 #define GFN_UNUSED	0xFFFFFFFFFFFFFFFFULL
 
 static bool guest64 (void);
-static void update_cr3 (void);
-static void invalidate_page (ulong virtual_addr);
-static void init_global (void);
-static int init_vcpu (void);
 
 /* key:
    bit 63-12: gfn
@@ -1156,8 +1152,9 @@ invalidate_page (ulong v)
 	pmap_close (&p);
 }
 
-static void
-update_cr3 (void)
+/* this function is called when a guest sets CR3 */
+void
+cpu_mmu_spt_updatecr3 (void)
 {
 	pmap_t p;
 	ulong cr0, cr3, cr4;
@@ -1209,8 +1206,10 @@ update_cr3 (void)
 	rw_spinlock_unlock_ex (&cspt->shadow2_lock);
 }
 
-static bool
-spt_tlbflush (void)
+/* this function is called when shadow page table entries are cleared
+   from TLB. every VM exit in VT. */
+bool
+cpu_mmu_spt_tlbflush (void)
 {
 	return update_rwmap (current->spt.data, 0, NULL, 0);
 }
@@ -1240,14 +1239,14 @@ spt_status (void)
 }
 
 static void
-init_global (void)
+cpu_mmu_spt_init_global (void)
 {
 	LIST1_HEAD_INIT (list1_spt);
 	register_status_callback (spt_status);
 }
 
-static int
-init_vcpu (void)
+void
+cpu_mmu_spt_init (void)
 {
 	int i;
 	struct sptlist *listspt;
@@ -1289,7 +1288,6 @@ init_vcpu (void)
 	listspt = alloc (sizeof *listspt);
 	listspt->spt = cspt;
 	LIST1_PUSH (list1_spt, listspt);
-	return 0;
 }
 
 static bool
@@ -1303,21 +1301,6 @@ guest64 (void)
 	return false;
 }
 
-/* this function is called when shadow page table entries are cleared
-   from TLB. every VM exit in VT. */
-bool
-cpu_mmu_spt_tlbflush (void)
-{
-	return spt_tlbflush ();
-}
-
-/* this function is called when a guest sets CR3 */
-void
-cpu_mmu_spt_updatecr3 (void)
-{
-	update_cr3 ();
-}
-
 /* this function is called by INVLPG in a guest */
 /* deletes TLB entry regardless of G bit */
 void
@@ -1435,16 +1418,4 @@ cpu_mmu_spt_pagefault (ulong err, ulong cr2)
 	map_page (cr2, m1, m2, gfns, levels, hphys);
 }
 
-static void
-cpu_mmu_spt_init_global (void)
-{
-	init_global ();
-}
-
-void
-cpu_mmu_spt_init (void)
-{
-	init_vcpu ();
-}
-
 INITFUNC ("global4", cpu_mmu_spt_init_global);
