@@ -38,6 +38,7 @@
 #include <core/printf.h>
 #include <core/string.h>
 #include <io/pci.h>
+#include "apic.h"
 #include "apic_pass.h"
 #include "callrealmode.h"
 #include "convert.h"
@@ -64,6 +65,7 @@
 #include "vt.h"
 
 static struct multiboot_info multiboot_info;
+extern char *build_date;
 
 static void
 print_boot_msg (void)
@@ -76,6 +78,7 @@ print_boot_msg (void)
 	printf ("Copyright (c) 2007, 2008 University of Tsukuba\n");
 	printf ("Copyright (c) 2010-2014 Yuichi Watanabe\n");
 	printf ("All rights reserved.\n");
+	printf ("Build at %s.\n", build_date);
 }
 
 static void
@@ -239,6 +242,13 @@ setup_vm (void)
 
 	sync_all_processors ();
 	vcpu_reset();
+	if (vm_get_id() != 0) {
+		/*
+		 * Mask local apic's interrupts, because VMM does not
+		 * allow VMs except VM0 to use lint0 and lint1.
+		 */
+		apic_maskall();
+	}
 	if (cpu_is_bsp()) {
 		setup_guestboot();
 	} else if (vbsp) {
