@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Copyright (c) 2010-2012 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  */
 
 #include <core/assert.h>
@@ -43,6 +43,7 @@
 #include "current.h"
 #include "entry.h"
 #include "mm.h"
+#include "msr_pass.h"
 #include "panic.h"
 #include "vramwrite.h"
 
@@ -58,7 +59,8 @@
 extern u8 end[];
 
 static phys_t memorysize = 0;
-static phys32_t top_of_low_avail_mem;
+static phys32_t top_of_low_avail_mem = 0;
+static phys_t top_of_avail_mem = 0;
 static phys32_t vmm_start_phys;
 static struct page pagestruct[NUM_OF_PAGES];
 static spinlock_t mapmem_lock;
@@ -67,6 +69,7 @@ static struct resource mem_address_space;
 static struct resource sysmemmap[MAXNUM_OF_SYSMEMMAP];
 static struct resource vmm_mem_resource;
 static int sysmemmaplen;
+static phys_t num_of_hphys_pages = 0;
 
 struct memsizetmp {
 	void *addr;
@@ -91,6 +94,12 @@ get_phys_mem_map(int index, phys_t *base, phys_t *len, u32 *type)
 	return 1;
 }
 
+phys_t
+mm_top_of_avail_mem()
+{
+	return top_of_avail_mem;
+}
+
 phys32_t
 mm_top_of_low_avail_mem()
 {
@@ -125,6 +134,9 @@ getallsysmemmap (void)
 
 		if (entry->data == MEM_TYPE_AVAILABLE) {
 			memsize += e820map.len;
+			if (entry->end > top_of_avail_mem) {
+				top_of_avail_mem = entry->end;
+			}
 			if (entry->end <= 0xffffffff &&
 			    entry->end > top_of_low_avail_mem) {
 				top_of_low_avail_mem = entry->end;
@@ -302,7 +314,12 @@ move_vmm (void)
 #endif
 
 #ifdef __x86_64__
-	move_vmm_area64 ();
+	/*
+	 * If I call move_vmm_area64, a machine is reset.
+	 * If I call memcpy, a machine continue to work.
+	 */
+	/* move_vmm_area64 (); */
+	memcpy((char *)0xC0100000L, (char *)0x40100000L, end - head);
 #else
 	move_vmm_area32 ();
 #endif
@@ -320,6 +337,11 @@ map_hphys (void)
 	int pdpi, pdi;
 	vmmerr_t err;
 
+	num_of_hphys_pages = mm_top_of_avail_mem() >> PAGE_SHIFT;
+	if (num_of_hphys_pages > 512 * 512 * 512) {
+		num_of_hphys_pages = 512 * 512 * 512;
+	}
+
 	err = alloc_page (&virt, &phys);
 	if (err) {
 		panic("Failed to allocate memory for a l3 page table.");
@@ -330,10 +352,10 @@ map_hphys (void)
 	pdp = virt;
 	pdpi = 0;
 	pdi = 512;
-	for (i = 0; i < NUM_OF_HPHYS_PAGES; i += 512) {
+	for (i = 0; i < num_of_hphys_pages; i += 512) {
 		if (pdi >= 512) {
 			if (pdpi >= 512)
-				panic ("NUM_OF_HPHYS_PAGES is too large.");
+				panic ("num_of_hphys_pages is too large.");
 			err = alloc_page (&virt, &phys);
 			if (err) {
 				panic("Failed to allocate memory for a l2 page table.");
@@ -385,6 +407,8 @@ mm_init_global (void)
 		      err);
 	}
 
+	vramwrite_exit_global();
+
 	move_vmm ();
 	mm_alloc_init ();
 	for (i = 0; i < NUM_OF_PAGES; i++) {
@@ -496,9 +520,11 @@ pmap_rd32 (pmap_t *m, u64 phys, u32 attr)
 		r = *(u32 *)phys_to_virt (phys);
 		break;
 	case PMAP_TYPE_GUEST:
+		attr = msr_pte_to_cache_flag(attr);
 		read_gphys_l (phys, &r, attr);
 		break;
 	case PMAP_TYPE_GUEST_ATOMIC:
+		attr = msr_pte_to_cache_flag(attr);
 		cmpxchg_gphys_l (phys, &r, r, attr);
 		break;
 	}
@@ -515,9 +541,11 @@ pmap_rd64 (pmap_t *m, u64 phys, u32 attr)
 		r = *(u64 *)phys_to_virt (phys);
 		break;
 	case PMAP_TYPE_GUEST:
+		attr = msr_pte_to_cache_flag(attr);
 		read_gphys_q (phys, &r, attr);
 		break;
 	case PMAP_TYPE_GUEST_ATOMIC:
+		attr = msr_pte_to_cache_flag(attr);
 		cmpxchg_gphys_q (phys, &r, r, attr);
 		break;
 	}
@@ -535,9 +563,11 @@ pmap_wr32 (pmap_t *m, u64 phys, u32 attr, u64 oldentry, u64 *entry)
 		*(u32 *)phys_to_virt (phys) = *entry;
 		break;
 	case PMAP_TYPE_GUEST:
+		attr = msr_pte_to_cache_flag(attr);
 		write_gphys_l (phys, *entry, attr);
 		break;
 	case PMAP_TYPE_GUEST_ATOMIC:
+		attr = msr_pte_to_cache_flag(attr);
 		tmp = oldentry;
 		r = cmpxchg_gphys_l (phys, &tmp, *entry, attr);
 		if (r)
@@ -557,9 +587,11 @@ pmap_wr64 (pmap_t *m, u64 phys, u32 attr, u64 oldentry, u64 *entry)
 		*(u64 *)phys_to_virt (phys) = *entry;
 		break;
 	case PMAP_TYPE_GUEST:
+		attr = msr_pte_to_cache_flag(attr);
 		write_gphys_q (phys, *entry, attr);
 		break;
 	case PMAP_TYPE_GUEST_ATOMIC:
+		attr = msr_pte_to_cache_flag(attr);
 		r = cmpxchg_gphys_q (phys, &oldentry, *entry, attr);
 		if (r)
 			*entry = oldentry;
@@ -725,23 +757,21 @@ pmap_dump (pmap_t *m)
 static void *
 mapped_hphys_addr (phys_t hphys, uint len)
 {
-#if NUM_OF_HPHYS_PAGES == 0
-	return NULL;
-#else
-	if ((hphys >> PAGESIZE_SHIFT) >= NUM_OF_HPHYS_PAGES)
+#ifdef __x86_64__
+	if ((hphys >> PAGESIZE_SHIFT) >= num_of_hphys_pages)
 		return NULL;
-	if (((hphys + len - 1) >> PAGESIZE_SHIFT) >= NUM_OF_HPHYS_PAGES)
+	if (((hphys + len - 1) >> PAGESIZE_SHIFT) >= num_of_hphys_pages)
 		return NULL;
 	return (void *)(virt_t)(HPHYS_ADDR + hphys);
+#else
+	return NULL
 #endif
 }
 
 static void *
 mapped_gphys_addr(phys_t gphys, uint len, u32 flags)
 {
-#if NUM_OF_HPHYS_PAGES == 0
-	return NULL;
-#else
+#ifdef __x86_64__
 	phys_t p1, p2;
 	phys_t hphys, hphys1, hphys2;
 
@@ -763,6 +793,8 @@ mapped_gphys_addr(phys_t gphys, uint len, u32 flags)
 		}
 	}
 	return mapped_hphys_addr(hphys, len);
+#else
+	return NULL
 #endif
 }
 
@@ -846,7 +878,7 @@ mapmem_domap(pmap_t *m, void *virt_addr, int flags, phys_t phys_addr, uint len)
 			hphys = current->vm->gmm.gp2hp(phys);
 			if (hphys == GMM_NO_MAPPING) {
 				printf("Failed to map no gp2hp mapping area. cpu %d gphys 0x%llx\n",
-				       get_cpu_id(), hphys);
+				       get_cpu_id(), phys);
 				goto unmap;
 			}
 		} else {
