@@ -355,8 +355,9 @@ vpanic(struct int_stack *stack, const char *format, va_list ap)
 
 	if (currentcpu_available()) {
 		if (currentcpu->panic) {
-			printf("The second panic on CPU%d: \n", get_cpu_id());
+			printf("The second panic on CPU%d: ", get_cpu_id());
 			vprintf(format, ap);
+			printf("\n");
 			freeze();
 		};
 		currentcpu->panic = true;
