@@ -434,10 +434,13 @@ svm_mainloop (void)
 void
 svm_invlpg (ulong addr)
 {
-	if (!current->u.svm.np)
+	if (!current->u.svm.np) {
 		cpu_mmu_spt_invalidate (addr);
-	else
+		current->u.svm.vi.vmcb->tlb_control
+			= VMCB_TLB_CONTROL_FLUSH_TLB;
+	} else {
 		panic ("invlpg while np enabled");
+	}
 }
 
 void
