@@ -234,10 +234,7 @@ pe_change_enable_sw (bool pe)
 
 	/* enable emulation */
 	vr->sw.num = 0; /* number of emulated instructions */
-	vr->sw.enable
-		= SW_SREG_ES_BIT | SW_SREG_CS_BIT
-		| SW_SREG_SS_BIT | SW_SREG_DS_BIT
-		| SW_SREG_FS_BIT | SW_SREG_GS_BIT;
+	vr->sw.enable = 0;
 
 	if (pe) {
 		/* real address mode to protected mode */
@@ -262,21 +259,32 @@ pe_change_enable_sw (bool pe)
 		asm_vmwrite (VMCS_GUEST_DS_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_FS_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_GS_ACCESS_RIGHTS, 0xF3);
+
+		asm_vmwrite (VMCS_GUEST_SS_SEL, 8);
 		/*
-		 * If a selector of CS or SS is a multiple of 4,
-		 * it is usable as it is without emulation
-		 * A null selector is also acceptable
+		 * If a RPL of CS or SS is 0, it is usable as it is
+		 * without emulation. A null selector is also acceptable.
+		 * If a RPL is not 0, set segment selector to dummy value
+		 * (RPL 0, TI 0, Index 1), and emulate it.
 		 */
 		if ((vr->sw.cs & 3) == 0) {
 			asm_vmwrite (VMCS_GUEST_CS_SEL, vr->sw.cs);
-			vr->sw.enable &= ~SW_SREG_CS_BIT;
+		} else {
+			vr->sw.enable |= SW_SREG_CS_BIT;
+			asm_vmwrite (VMCS_GUEST_CS_SEL, 8);
 		}
+
 		if ((vr->sw.ss & 3) == 0) {
 			asm_vmwrite (VMCS_GUEST_SS_SEL, vr->sw.ss);
-			vr->sw.enable &= ~SW_SREG_SS_BIT;
+		} else {
+			asm_vmwrite (VMCS_GUEST_CS_SEL, 8);
+			vr->sw.enable |= SW_SREG_SS_BIT;
 		}
-		vr->sw.enable &= ~(SW_SREG_ES_BIT | SW_SREG_DS_BIT
-				   | SW_SREG_FS_BIT | SW_SREG_GS_BIT);
+
+		asm_vmwrite (VMCS_GUEST_ES_SEL, vr->sw.es);
+		asm_vmwrite (VMCS_GUEST_DS_SEL, vr->sw.ds);
+		asm_vmwrite (VMCS_GUEST_FS_SEL, vr->sw.fs);
+		asm_vmwrite (VMCS_GUEST_GS_SEL, vr->sw.gs);
 	} else {
 		/* protected mode to real address mode */
 
@@ -288,58 +296,40 @@ pe_change_enable_sw (bool pe)
 		/*
 		 * If the guest is in virtual-8086 mode, base-address
 		 * fields must be the selector field shifted left 4
-		 * bits. BASE fields must be < 0xfffff.
+		 * bits.
 		 */
 		/* FIXME: segment base must be a multiple of 16.
 		   if not, it will be panic. */
 		asm_vmread (VMCS_GUEST_CS_BASE, &tmp);
 		asm_vmwrite (VMCS_GUEST_CS_SEL, tmp >> 4);
-		if ((tmp >> 4) == vr->sw.cs)
-		       vr->sw.enable &= ~SW_SREG_CS_BIT;
+		if ((tmp >> 4) != vr->sw.cs)
+		       vr->sw.enable |= SW_SREG_CS_BIT;
+
 		asm_vmread (VMCS_GUEST_ES_BASE, &tmp);
-		if (tmp > 0xfffff) {
-			tmp = 0;
-			asm_vmwrite (VMCS_GUEST_ES_BASE, tmp);
-		}
 		asm_vmwrite (VMCS_GUEST_ES_SEL, tmp >> 4);
-		if ((tmp >> 4) == vr->sw.es)
-			vr->sw.enable &= ~SW_SREG_ES_BIT;
+		if ((tmp >> 4) != vr->sw.es)
+			vr->sw.enable |= SW_SREG_ES_BIT;
 
 		asm_vmread (VMCS_GUEST_SS_BASE, &tmp);
-		if (tmp > 0xfffff) {
-			tmp = 0;
-			asm_vmwrite (VMCS_GUEST_SS_BASE, tmp);
-		}
 		asm_vmwrite (VMCS_GUEST_SS_SEL, tmp >> 4);
-		if ((tmp >> 4) == vr->sw.ss)
-			vr->sw.enable &= ~SW_SREG_SS_BIT;
+		if ((tmp >> 4) != vr->sw.ss)
+			vr->sw.enable |= SW_SREG_SS_BIT;
 
 		asm_vmread (VMCS_GUEST_DS_BASE, &tmp);
-		if (tmp > 0xfffff) {
-			tmp = 0;
-			asm_vmwrite (VMCS_GUEST_DS_BASE, tmp);
-		}
 		asm_vmwrite (VMCS_GUEST_DS_SEL, tmp >> 4);
-		if ((tmp >> 4) == vr->sw.ds)
-			vr->sw.enable &= ~SW_SREG_DS_BIT;
+		if ((tmp >> 4) != vr->sw.ds)
+			vr->sw.enable |= SW_SREG_DS_BIT;
 
 		asm_vmread (VMCS_GUEST_FS_BASE, &tmp);
-		if (tmp > 0xfffff) {
-			tmp = 0;
-			asm_vmwrite (VMCS_GUEST_FS_BASE, tmp);
-		}
 		asm_vmwrite (VMCS_GUEST_FS_SEL, tmp >> 4);
-		if ((tmp >> 4) == vr->sw.fs)
-			vr->sw.enable &= ~SW_SREG_FS_BIT;
+		if ((tmp >> 4) != vr->sw.fs)
+			vr->sw.enable |= SW_SREG_FS_BIT;
 
 		asm_vmread (VMCS_GUEST_GS_BASE, &tmp);
-		if (tmp > 0xfffff) {
-			tmp = 0;
-			asm_vmwrite (VMCS_GUEST_GS_BASE, tmp);
-		}
 		asm_vmwrite (VMCS_GUEST_GS_SEL, tmp >> 4);
-		if ((tmp >> 4) == vr->sw.gs)
-			vr->sw.enable &= ~SW_SREG_GS_BIT;
+		if ((tmp >> 4) != vr->sw.gs)
+			vr->sw.enable |= SW_SREG_GS_BIT;
+
 		/*
 		 * If the guest will be virtual-8086, limit fields
 		 * must be 0000FFFFH.
@@ -389,6 +379,7 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 	ulong cr0_0, cr0_1;
 	ulong cr4_0, cr4_1;
 	ulong host_val;
+	ulong old_val;
 	ulong cr4;
 
 	switch (reg) {
@@ -402,7 +393,6 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 		if (!!(val & CR0_PG_BIT) != !!current->u.vt.vr.pg) {
 			current->u.vt.vr.pg = !!(val & CR0_PG_BIT);
 			vt_ept_pg_change(current->u.vt.vr.pg);
-
 			/*
 			 * Update actual guest cr4.
 			 */
@@ -424,18 +414,29 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 		current->u.vt.vr.cr2 = val;
 		break;
 	case CONTROL_REG_CR3:
-		current->u.vt.vr.cr3 = val;
+		current->u.vt.vr.cr3 = val & ~CR3_NO_INVALIDATE;
 		if (current->u.vt.ept_enabled && current->u.vt.vr.pg) {
-			asm_vmwrite(VMCS_GUEST_CR3, val);
+			asm_vmwrite(VMCS_GUEST_CR3, val & ~CR3_NO_INVALIDATE);
+		} else {
+			cpu_mmu_spt_updatecr3();
+		}
+		vt_read_control_reg(CONTROL_REG_CR3, &cr4);
+		if ((cr4 & CR4_PCIDE_BIT) == 0) {
+			vt_invalidate_tlb();
+		} else if ((val & CR3_NO_INVALIDATE) == 0) {
+			/*
+			 * We can't specify PCID with invept.
+			 * So all TLB will be invalidated with any PCIID.
+			 */
+			vt_invalidate_tlb();
 		}
-		cpu_mmu_spt_updatecr3();
 		break;
 	case CONTROL_REG_CR4:
 		/*
 		 *Update host cr4.
 		 */
 		asm_rdcr4(&host_val);
-		host_val = (host_val & ~CR4_PASSTHROUGH)
+		host_val = (host_val & ~CR4_PASSTHROUGH) /* Only OSXSAVE */
 			| (val & CR4_PASSTHROUGH);
 		asm_wrcr4(host_val);
 		asm_vmwrite(VMCS_HOST_CR4, host_val);
@@ -443,6 +444,7 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 		/*
 		 * Update guest cr4 shadow.
 		 */
+		asm_vmread (VMCS_CR4_READ_SHADOW, &old_val);
 		asm_vmwrite (VMCS_CR4_READ_SHADOW, val);
 
 		/*
@@ -470,7 +472,10 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 		}
 		asm_vmwrite(VMCS_GUEST_CR4, val);
 
-		cpu_mmu_spt_updatecr3();
+		if (old_val ^ val) {
+			vt_invalidate_tlb();
+			cpu_mmu_spt_updatecr3();
+		}
 		break;
 	default:
 		panic ("Unknown control register.");
