@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Copyright (c) 2010-2013 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  */
 
 #ifndef __IO_PCI_H
@@ -153,7 +153,7 @@ struct idmask { u32 id, mask; };
 #define PCI_PM_CONTROL_STATUS_REG_POWER_D0	0x0
 #define PCI_PM_CONTROL_STATUS_REG_POWER_D3HOT	0x3
 #define PCI_PM_CONTROL_STATUS_REG_POWER_MASK	0x3
-
+#define PCI_PM_CONTROL_STATUS_REG_NO_SOFT_RESET	0x8
 
 #define PCI_RESOURCE_NUM			9
 #define PCI_RESOURCE_IO_WINDOW_INDEX		2
@@ -235,6 +235,7 @@ struct pci_driver {
 };
 
 void pci_register_driver (struct pci_driver *driver);
+void pci_register_driver_for_assigned_device(struct pci_driver *driver);
 struct pci_device *pci_next_assgined_pci_device(struct pci_device *cur);
 void pci_dump_resources(void);
 
