@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2012 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,29 +33,22 @@
  *   Not-supports injecting a level-triggered interrupt.
  */
 
-#include <core/cpu.h>
+#include <core/assert.h>
 #include <core/extint.h>
 #include <core/initfunc.h>
 #include <core/mmio.h>
 #include <core/printf.h>
 #include <core/spinlock.h>
+#include <core/string.h>
 #include <core/types.h>
 #include <core/vm.h>
 #include <core/vmmerr.h>
 #include <io/ioapic_emu.h>
 #include "hpet.h"
+#include "ioapic.h"
+#include "ioapic_regs.h"
 
-#define IOAPIC_MEM_START		0xfec00000
-#define IOAPIC_MEM_SIZE			0x00100000
-
-#define IOAPIC_INDEX_REG		0xfec00000
-#define IOAPIC_DATA_REG			0xfec00010
-
-#define IOAPIC_ID_REG_INDEX		0x00
-#define IOAPIC_VER_REG_INDEX		0x01
-#define IOAPIC_REDIR_TBL_SIZE		2
 #define IOAPIC_REDIR_TBL_NUM		24
-#define IOAPIC_REDIR_TBL0_LOW_INDEX	0x10
 #define IOAPIC_REDIR_TBLmax_HIGH_INDEX \
 	(IOAPIC_REDIR_TBL0_LOW_INDEX \
 	 + IOAPIC_REDIR_TBL_SIZE * IOAPIC_REDIR_TBL_NUM \
@@ -69,23 +62,12 @@
 	((index - IOAPIC_REDIR_TBL0_LOW_INDEX) / 2)
 
 #define IOAPIC_VER_REG_VAL			0x00170011
-#define IOAPIC_REDIR_LOW_MASK_BIT		0x00010000
-#define IOAPIC_REDIR_LOW_VECTOR_MASK		0x000000ff
-#define IOAPIC_REDIR_LOW_VECTOR_SHIFT		0
-#define IOAPIC_REDIR_LOW_DELIVERY_MODE_MASK	0x00000700
-#define IOAPIC_REDIR_LOW_DELIVERY_MODE_FIXED	0x00000000
-
-#define IOAPIC_REDIR_LOW_LOGICAL_DEST_BIT	0x00000800
-#define IOAPIC_REDIR_LOW_LEVEL_TRIGGER_BIT	0x00008000
-
-#define IOAPIC_REDIR_HIGH_DEST_MASK		0xff000000
-#define IOAPIC_REDIR_HIGH_DEST_SHIFT		24
 
 /* #define IOAPIC_DEBUG */
 #ifdef IOAPIC_DEBUG
 #define IOAPIC_DBG(...)						\
 	do {							\
-		printf(__VA_ARGS__);				\
+		printf("IOAPICEMU: " __VA_ARGS__);		\
 	} while (0)
 #else
 #define IOAPIC_DBG(...)
@@ -106,6 +88,8 @@ ioapic_get_vector(gsi_t gsi, vector_t *vector, bool *mask)
 	struct ioapic_emu	*ioapic;
 	u32			redir_low;
 
+	ASSERT(vm_get_id() != 0);
+
 	if (gsi >= IOAPIC_REDIR_TBL_NUM) {
 		return VMMERR_RANGE;
 	}
@@ -126,35 +110,42 @@ ioapic_trigger_int(gsi_t gsi)
 {
 	struct ioapic_emu	*ioapic;
 	vector_t		vector;
-	u32			redir_low;
+	u32			redir_low, redir_high;
 	apic_id_t		dest;
+	vmmerr_t		err;
 
-	if (gsi >= IOAPIC_REDIR_TBL_NUM) {
-		return;
+	if (vm_get_id() == 0) {
+		err = ioapic_read_redir(gsi, &redir_high, &redir_low);
+		if (err != VMMERR_SUCCESS) {
+			return;
+		}
+	} else {
+		if (gsi >= IOAPIC_REDIR_TBL_NUM) {
+			return;
+		}
+		ioapic = vm_get_driver_data(ioapic_handle);
+		spinlock_lock(&ioapic->lock);
+		redir_low = IOAPIC_REDIR_LOW(ioapic, gsi);
+		redir_high = IOAPIC_REDIR_HIGH(ioapic, gsi);
+		spinlock_unlock(&ioapic->lock);
 	}
 
-	ioapic = vm_get_driver_data(ioapic_handle);
-	spinlock_lock(&ioapic->lock);
-
-	redir_low = IOAPIC_REDIR_LOW(ioapic, gsi);
 	if (redir_low & (IOAPIC_REDIR_LOW_MASK_BIT |
-			 IOAPIC_REDIR_LOW_LEVEL_TRIGGER_BIT)) {
-		spinlock_unlock(&ioapic->lock);
+			 IOAPIC_REDIR_LOW_LEVEL_TRIGGER_BIT |
+			 IOAPIC_REDIR_LOW_LOGICAL_DEST_BIT)) {
 		return;
 	}
 	if ((redir_low & IOAPIC_REDIR_LOW_DELIVERY_MODE_MASK) !=
 	    IOAPIC_REDIR_LOW_DELIVERY_MODE_FIXED) {
-		spinlock_unlock(&ioapic->lock);
 		return;
 	}
 	vector = (redir_low & IOAPIC_REDIR_LOW_VECTOR_MASK)
 		>> IOAPIC_REDIR_LOW_VECTOR_SHIFT;
-	dest = (IOAPIC_REDIR_HIGH(ioapic, gsi) & IOAPIC_REDIR_HIGH_DEST_MASK)
+	dest = (redir_high & IOAPIC_REDIR_HIGH_DEST_MASK)
 		>> IOAPIC_REDIR_HIGH_DEST_SHIFT;
 
 	IOAPIC_DBG("IOAPIC: trigger vector 0x%x dest 0x%x\n", vector, dest);
 	extint_send_apic_int(vector, dest);
-	spinlock_unlock(&ioapic->lock);
 }
 
 static gsi_t
@@ -231,45 +222,52 @@ ioapic_mem_reg_handler(void *data, phys_t gphys, bool wr, void *buf,
 {
 	struct ioapic_emu *ioapic = (struct ioapic_emu *)data;
 	gsi_t updated_gsi = INVALID_GSI;;
-	int ret = 1;
+	phys_t offset = gphys - IOAPIC_MEM_START;
 	int done = 0;
+	u32 *buf32 = (u32 *)buf;
+	u8 *buf8 = (u8 *)buf;
 
 	spinlock_lock(&ioapic->lock);
 
-	if (len == 4) {
-		if (wr) {
-			switch (gphys) {
-			case IOAPIC_INDEX_REG:
-				ioapic->index = *(u32 *)buf;
-				done = 1;
+	if (wr) {
+		switch (offset) {
+		case IOAPIC_INDEX_REG_OFFSET:
+			ioapic->index = *buf8;
+			done = 1;
+			break;
+		case IOAPIC_DATA_REG_OFFSET:
+			if (len != 4) {
 				break;
-			case IOAPIC_DATA_REG:
-				updated_gsi = ioapic_write_data_reg(
-					ioapic, *(u32 *)buf);
-				done = 1;
-				break;
-			default:
-				IOAPIC_DBG("IOAPIC: Write unknown reg. gphys 0x%llx, val 0x%x\n",
-					   gphys, *(u32 *)buf);
 			}
-		} else {
-			switch (gphys) {
-			case IOAPIC_INDEX_REG:
-				*(u32 *)buf = ioapic->index;
-				done = 1;
+			updated_gsi = ioapic_write_data_reg(ioapic, *buf32);
+			done = 1;
+			break;
+		default:
+			IOAPIC_DBG("IOAPIC: Write unknown reg. offset 0x%llx, val 0x%x\n",
+				   offset, *(u32 *)buf);
+		}
+	} else {
+		switch (offset) {
+		case IOAPIC_INDEX_REG_OFFSET:
+			memset(buf, 0, len);
+			*buf8 = ioapic->index;
+			done = 1;
+			break;
+		case IOAPIC_DATA_REG_OFFSET:
+			if (len != 4) {
 				break;
-			case IOAPIC_DATA_REG:
-				*(u32 *)buf = ioapic_read_data_reg(ioapic);
-				done = 1;
-				break;
-			default:
-				IOAPIC_DBG("IOAPIC: Read unknown reg. gphys 0x%llx\n",
-					   gphys);
 			}
+			*buf32 = ioapic_read_data_reg(ioapic);
+			done = 1;
+			break;
+		default:
+			IOAPIC_DBG("IOAPIC: Read unknown reg. offset 0x%llx\n",
+				   offset);
 		}
 	}
+
 	if (! done) {
-		ret = mmio_do_nothing(data, gphys, wr, buf, len, flags);
+		mmio_do_nothing(data, gphys, wr, buf, len, flags);
 	}
 
 	spinlock_unlock(&ioapic->lock);
@@ -277,12 +275,13 @@ ioapic_mem_reg_handler(void *data, phys_t gphys, bool wr, void *buf,
 	if (updated_gsi != INVALID_GSI) {
 		hpet_ioapic_updated(updated_gsi);
 	}
-	return ret;
+
+	return 1; /* emulated */
 }
 
 
 static void
-ioapic_setup (void)
+ioapic_emu_setup (void)
 {
 	struct ioapic_emu	*ioapic;
 	int			i;
@@ -294,7 +293,7 @@ ioapic_setup (void)
 		IOAPIC_REDIR_LOW(ioapic, i) = IOAPIC_REDIR_LOW_MASK_BIT;
 	}
 
-	if (cpu_is_bsp()) {
+	if (vm_get_id() == 0) {
 		/*
 		 * vm0 is allowed to access ioapic directory.
 		 */
@@ -306,10 +305,10 @@ ioapic_setup (void)
 }
 
 static void
-ioapic_init(void)
+ioapic_emu_init(void)
 {
 	ioapic_handle = vm_alloc_driver_data(sizeof(struct ioapic_emu));
 }
 
-DRIVER_PASSVM(ioapic_setup);
-DRIVER_INIT(ioapic_init);
+DRIVER_SETUPVM(ioapic_emu_setup);
+DRIVER_INIT(ioapic_emu_init);
