@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2013 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,30 @@
 #define PCI_DUMMY_FUNC0_DEVICE_ID 0x0001
 
 /* #define NO_CLEAR_IOBITMAP */
+/* #define NO_SECONDARY_BUS_RESET */
+/* #define NO_D3HOT_D0_RESET */
+
+static LIST_DEFINE_HEAD(pci_driver_list);
+
+void
+pci_register_driver_for_assigned_device(struct pci_driver *driver)
+{
+	LIST_APPEND(pci_driver_list, driver);
+	return;
+}
+
+static void
+pci_call_driver(struct pci_device *dev)
+{
+	struct pci_driver *driver;
+
+	LIST_FOREACH (pci_driver_list, driver) {
+		if (idmask_match(dev->id, driver->id) &&
+		    idmask_match(dev->class_code, driver->class)) {
+			driver->new(dev);
+		}
+	}
+}
 
 static void
 pci_save_config(struct pci_device *dev)
@@ -119,6 +143,8 @@ pci_setup_assigned_device(struct pci_device *dev)
 		}
 	}
 
+	pci_call_driver(dev);
+
 	LIST2_FOREACH(dev->children, sibling, child) {
 		pci_setup_assigned_device(child);
 	}
@@ -514,6 +540,7 @@ pci_find_capability(struct pci_device *dev, u8 cap_id)
 static void
 pci_secondary_bus_reset(struct pci_device *dev)
 {
+#ifndef NO_SECONDARY_BUS_RESET
 	u16 reg;
 
 	if (dev->type != PCI_CONFIG_HEADER_TYPE_1) {
@@ -529,6 +556,7 @@ pci_secondary_bus_reset(struct pci_device *dev)
 
 	reg &= ~PCI_CONFIG_SECONDARY_BUS_RESET;
 	pci_write_config16(dev, PCI_CONFIG_BRIDGE_CONTROL, reg);
+#endif /* NO_SECONDARY_BUS_RESET */
 }
 
 static void
@@ -545,21 +573,45 @@ pci_d3d0_reset_device(struct pci_device *dev)
 	if (base == 0) {
 		return;
 	}
-	printf("Reset %s\n", dev->name);
 
 	reg = pci_read_config16(dev, base + PCI_PM_CONTROL_STATUS_REG_OFF);
-	reg = (reg & ~PCI_PM_CONTROL_STATUS_REG_POWER_MASK)
-		| PCI_PM_CONTROL_STATUS_REG_POWER_D3HOT;
-	pci_write_config16(dev, base + PCI_PM_CONTROL_STATUS_REG_OFF, reg);
 
-	/* wait 10 ms */
+#ifndef NO_D3HOT_D0_RESET
+	if ((reg & PCI_PM_CONTROL_STATUS_REG_NO_SOFT_RESET) == 0) {
+		printf("Reset %s\n", dev->name);
+
+		reg = (reg & ~PCI_PM_CONTROL_STATUS_REG_POWER_MASK)
+			| PCI_PM_CONTROL_STATUS_REG_POWER_D3HOT;
+		pci_write_config16(dev,
+				   base + PCI_PM_CONTROL_STATUS_REG_OFF,
+				   reg);
+	}
+
+	/*wait 10 ms */
 	usleep(10 * 1000);
+#endif /* NO_D3HOT_D0_RESET */
 
 	reg = (reg & ~PCI_PM_CONTROL_STATUS_REG_POWER_MASK)
 		| PCI_PM_CONTROL_STATUS_REG_POWER_D0;
 	pci_write_config16(dev, base + PCI_PM_CONTROL_STATUS_REG_OFF, reg);
 }
 
+static void
+pci_d3d0_reset_devices(struct pci_device *dev)
+{
+	struct pci_device *child;
+	bool has_child = false;
+
+	LIST2_FOREACH(dev->children, sibling, child) {
+		pci_d3d0_reset_devices(child);
+		has_child = true;
+	}
+
+	if (!has_child) {
+		pci_d3d0_reset_device(dev);
+	}
+}
+
 static void
 pci_reprogram_device(struct pci_device *dev)
 {
@@ -605,7 +657,14 @@ pci_reset_assigned_device(void)
 	/* Secondary bus reset */
 	LIST2_FOREACH(pci_data->vm_pci_device_list, vm_pci_device_list, dev) {
 		pci_secondary_bus_reset(dev);
-		pci_d3d0_reset_device(dev);
+	}
+
+	/* wait 2 sec */
+	usleep(2 * 1000 * 1000);
+
+	/* D3hot D0 reset */
+	LIST2_FOREACH(pci_data->vm_pci_device_list, vm_pci_device_list, dev) {
+		pci_d3d0_reset_devices(dev);
 	}
 
 	/* wait 2 sec */
