@@ -315,8 +315,7 @@ parse_dmar(void *table, u32 length)
 	}
 	
 	printf("ACPI DMAR cleared.\n");
-	// Zap APCI 'DMAR' signature to avoid guest OS's detection
-	memset(dmar, 0, 4);
+	acpi_hide_table(table);
 	return;
 }
 
